#include "example.h"

static void usage( char *pgmName );

#define OPT_ARG_LIST        "hd:i:s:"

/*
 * if not supplied, how many seconds to wait for heartbeat
 */
#define DEFAULT_HBEATWAIT_SEC   5

/* 
 * for ultra sparc .. appears that it's
 * 100 ticks per second in default configuration
 */
#define TICKS_PER_SEC       100                  

/********************************************************
**
** Test Module: WaitHeartbeat.c
**
** Purpose:     Wait for a heartbeat interrupt using
**              a parameter-supplied number of seconds
**              timeout value.
**
*********************************************************/

int main (int argc, char *argv[])
{
    TPRO_BoardObj   *hnd = NULL;
    int             rc;
    int             ret = 0;
    int             devIdx = DEFAULT_DEV_NUM;
    char            *devName = DEFAULT_DEV;
    int             optRc;
    char            fullDevName[MAX_DEVICE_PATH_LENGTH];
    char            *errMsg;
    char            *pgmname = argv[0];
    int             secToWait = DEFAULT_HBEATWAIT_SEC;

    opterr = 0;
    while ( (optRc = getopt (argc, argv, OPT_ARG_LIST)) != -1 )
    {
        switch ( optRc )
        {
            case 'd' :
                devName = optarg;
                break;
            case 'i' :
                devIdx = atoi(optarg);
                break;
            case 's' :
                secToWait = atoi(optarg);
                break;
            case 'h' :
            default:
                usage( pgmname );
                return (1);
        }
    }

    /*
     * If leftover args, then command is misunderstood...
     * so let user know.
     */
    if ( optind != argc )
    {
        usage( pgmname );
        return (1);
    }

    /*
     * construct the device name
     */
    (void)sprintf_s( fullDevName, MAX_DEVICE_PATH_LENGTH, "%s%s%d", DEVICE_ROOT, devName, devIdx );

    /*
     * open the device
     */
    if ( (rc = TPRO_open(&hnd, fullDevName)) != TPRO_SUCCESS )
    {
        /* 
         * if errors, close it
         */
        printf("%s: Could not open <%s> : rc <%d>\n", 
                        pgmname, fullDevName, rc);
        return (1);
    }

    /* 
     * Wait for a heartbeat.
     */
    (void) printf ("Waiting on heartbeat interrupt...");
    fflush (stdout);

    /*
     * Set the timeout value in clock ticks and then
     * issue the call to wait.
     */
    unsigned int ticks = ( secToWait  * TICKS_PER_SEC );

    if ( (rc = TPRO_waitHeartbeat(hnd, &ticks)) != TPRO_SUCCESS )
    {
        char emsg[MAX_ERR_MESG_LENGTH];

        switch (rc)
        {
            case TPRO_TIMEOUT_ERR:
                (void)sprintf_s(emsg, MAX_ERR_MESG_LENGTH, " TIMEOUT_ERROR <%d> ticks ", ticks );
                errMsg = emsg;
                break;
            case TPRO_HANDLE_ERR:
                errMsg = "TPRO_HANDLE_ERR";
                break;
            case TPRO_COMM_ERR:
                errMsg = "TPRO_COMM_ERR";
                break;
            default:
                errMsg = "Unknown error";
                break;
        }

        (void) printf("\n%s: <%s> %s rc: <%d>.\n", 
                        pgmname, fullDevName, errMsg, rc);

        ret = 1;
    }
    else
    {
        /*
         * success notification
         */
        (void)printf("<%s>  received Heartbeat.\n", fullDevName ); 
    }

    /*
     * close the handle to the board
     */
    if ( (rc = TPRO_close(hnd)) != TPRO_SUCCESS )
    {
        (void) printf("%s: error closing <%s> rc: <%d>\n", 
                pgmname, fullDevName, rc );
        ret = 1;
    }

    return (ret);
}

void usage( char *pgmName )
{
    (void)printf("Usage: %s [-s SECONDS] [-h] [-d NAME] [-i INDEX] \n\n", pgmName );
    (void)printf("\t-s   # of seconds to wait for heartbeat (default %d)\n", DEFAULT_HBEATWAIT_SEC);
    (void)printf("\t-h   help message\n");
    (void)printf("\t-d   device name (e.g. %s)\n", DEFAULT_DEV);
    (void)printf("\t-i   device index (e.g. 0,1,..)\n");
    (void)printf("\t\tsuch that combined forms: %s0\n\n", DEFAULT_DEV);
}

